import time
from typing import Any, Dict, Literal, Optional

from pydantic import BaseModel, EmailStr

CvsMapSubTypeOptions = Literal[
    "FAMI", "UNIMART", "FAMIC2C", "UNIMARTC2C", "HILIFE", "HILIFEC2C", "OKMARTC2C"
]

LogisticsTypeOptions = Literal["CVS", "Home"]
LogisticsStatusOptions = Literal[
    "pending",
    "in_delivery",
    "delivered",
    "exception",
    "center_delivered",
    "store_delivered",
]

LogisticSubTypeOptions = Literal[
    "FAMI",
    "UNIMART",
    "HILIFE",
    "FAMIC2C",
    "UNIMARTC2C",
    "HILIFEC2C",
    "OKMARTC2C",
    "TCAT",
    "ECAN",
]

DistanceOptions = Literal["00", "01", "02"]
TemperatureOptions = Literal["0001", "0002", "0003"]
SpecificationOptions = Literal["0001", "0002", "0003", "0004"]
ScheduledPickupTimeOptions = Literal["1", "2", "3", "4"]
ScheduledDeliveryTimeOptions = Literal["1", "2", "3", "4", "12", "13", "23"]


class CVSMapModel(BaseModel):
    LogisticsType: Literal["CVS"] = "CVS"
    LogisticsSubType: CvsMapSubTypeOptions = "FAMI"
    IsCollection: Literal["Y", "N"]
    Device: int = 0
    redirect_url: Optional[str]


class LogisticModel(BaseModel):
    MerchantTradeNo: str
    LogisticsType: LogisticsTypeOptions = "Home"
    LogisticsSubType: LogisticSubTypeOptions = "TCAT"
    GoodsName: Optional[str]
    GoodsAmount: int
    SenderName: str
    SenderPhone: str
    SenderCellPhone: Optional[str]
    ReceiverName: str
    ReceiverCellPhone: str
    ReceiverEmail: EmailStr
    IsCollection: Optional[bool]
    CollectionAmount: Optional[int]
    ReceiverStoreID: Optional[str]
    TradeDesc: Optional[str]
    Remark: Optional[str]

    # if LogisticsType = Home
    Distance: Optional[DistanceOptions]
    Temperature: Optional[TemperatureOptions]
    Specification: Optional[SpecificationOptions]
    ScheduledPickupTime: Optional[ScheduledPickupTimeOptions]
    ScheduledDeliveryTime: Optional[ScheduledDeliveryTimeOptions]
    ScheduledDeliveryDate: Optional[str]
    PackageCount: Optional[int]
    SenderZipCode: Optional[str]
    SenderAddress: Optional[str]
    ReceiverZipCode: Optional[str]
    ReceiverAddress: Optional[str]

    # only C2C
    ReturnStoreID: Optional[str]
    LogisticsC2CReplyURL: Optional[str]


class CallbackModel(BaseModel):
    logistics_status: LogisticsStatusOptions = "pending"
    logistics_type: str
    logistics_subtype: str
    logistics_id: str
    detail: Dict[str, Any]

    @property
    def rtn_random(self) -> str:
        rtn_code = self.detail.get("RtnCode", "ERR")
        return f"{rtn_code}_{int(time.time())}"

    @property
    def stepfn_name(self) -> str:
        return f"{self.logistics_id}_{self.logistics_status}_{self.rtn_random}"



from typing import Any, List

from fastel.config import SdkConfig
from fastel.utils import requests


def validation_request(issuer: str, identity: str) -> Any:
    url = f"{SdkConfig.auth_host}/validation/request?client_id={SdkConfig.client_id}&client_secret={SdkConfig.client_secret}&issuer={issuer}"
    result = requests.post(
        url,
        json={"identity": identity},
    )
    return result.json()


def gen_token_request(identity: str, group: List[str] = []) -> Any:
    url = f"{SdkConfig.auth_host}/jwt/server/encode?client_id={SdkConfig.client_id}&client_secret={SdkConfig.client_secret}"
    result = requests.post(
        url,
        json={"id": identity, "group": group},
    )
    return result.json()


def revoke_token_request(identity: str) -> Any:
    url = f"{SdkConfig.auth_host}/jwt/revoke?client_id={SdkConfig.client_id}&client_secret={SdkConfig.client_secret}"
    result = requests.post(url, json={"sub": identity})
    return result.json()



import time
from typing import Any, List, Optional

from facebook_business.adobjects.serverside.action_source import ActionSource
from facebook_business.adobjects.serverside.content import Content
from facebook_business.adobjects.serverside.custom_data import CustomData
from facebook_business.adobjects.serverside.event import Event
from facebook_business.adobjects.serverside.event_request import EventRequest
from facebook_business.adobjects.serverside.user_data import UserData
from facebook_business.api import FacebookAdsApi
from pydantic import BaseModel

from fastel.config import SdkConfig


class UserForm(BaseModel):
    email: str
    phone: Optional[str]
    city: Optional[str]
    state: Optional[str]


class ItemForm(BaseModel):
    product_id: str
    quantity: int
    item_price: int
    title: str
    category: Optional[str]


class ItemsForm(BaseModel):
    order_id: Optional[str] = None
    price: Optional[int] = None
    content: List[ItemForm]


class ConversionApi:
    conversion_id: str

    @classmethod
    def init(cls) -> Any:
        cls.conversion_id = SdkConfig.conversion_id
        FacebookAdsApi.init(access_token=SdkConfig.conversion_token)

    @classmethod
    def push_event(
        cls,
        action: str,
        user_model: UserForm,
        custom_model: Optional[ItemsForm] = None,
        raise_exception: bool = False,
    ) -> Any:
        try:
            user_data = UserData(**user_model.dict(exclude_none=True))

            custom_data = None
            if custom_model:
                contents = [
                    Content(**item.dict(exclude_none=True))
                    for item in custom_model.content
                ]
                custom_data = CustomData(
                    currency="twd",
                    order_id=custom_model.order_id,
                    value=custom_model.price,
                    contents=contents,
                )

            event = Event(
                event_name=action,
                event_time=int(time.time()),
                user_data=user_data,
                custom_data=custom_data,
                action_source=ActionSource.WEBSITE,
            )
            events = [event]
            event_request = EventRequest(
                events=events,
                pixel_id=cls.conversion_id,
            )
            event_result = event_request.execute()
            print("[SUCCESS]", event_result)
            return event_result

        except Exception as exc:
            print("[ERROR]", str(exc))
            if raise_exception:
                raise exc

    @classmethod
    def push_register_event(cls, user: UserForm) -> Any:
        cls.push_event("CompleteRegistration", user, None)

    @classmethod
    def push_add_cart_event(cls, user: UserForm, items: ItemsForm) -> Any:
        cls.push_event("AddToCart", user, items)

    @classmethod
    def push_checkout_event(cls, user: UserForm, items: ItemsForm) -> Any:
        cls.push_event("InitiateCheckout", user, items)

    @classmethod
    def push_purchase_event(cls, user: UserForm, items: ItemsForm) -> Any:
        cls.push_event("Purchase", user, items)



from typing import Any, Literal

from pydantic import BaseModel

from fastel.config import SdkConfig
from fastel.utils import requests


class MapRequestModel(BaseModel):
    logistics_subtype: Literal[
        "FAMI",
        "UNIMART",
        "HILIFE",
        "FAMIC2C",
        "UNIMARTC2C",
        "HILIFEC2C",
        "OKMARTC2C",
        "TCAT",
        "ECAN",
    ]
    redirect_url: str


def map_request(logistics_subtype: str, redirect_url: str) -> Any:
    data = {
        "LogisticsType": "CVS",
        "LogisticsSubType": logistics_subtype,
        "IsCollection": "N",
        "Device": 0,
    }
    if redirect_url:
        data["redirect_url"] = redirect_url

    url = f"{SdkConfig.payment_host}/ecpay/logistics/map/request?client_id={SdkConfig.client_id}&client_secret={SdkConfig.client_secret}"
    result = requests.post(url, json=data)
    return result.json()



import base64
import hashlib
import os


def _encrypt(salt: bytes, password: str) -> bytes:
    key = hashlib.pbkdf2_hmac("sha256", password.encode(), salt, 100000)
    return salt + key


def password_encode(password: str) -> str:
    salt = os.urandom(32)
    password_encoded = _encrypt(salt=salt, password=password)
    stored_password = base64.b64encode(password_encoded).decode()
    return stored_password


def password_verify(stored: str, password: str) -> bool:
    stored_password = base64.b64decode(stored.encode())
    salt = stored_password[:32]
    val_password = _encrypt(salt, password)
    return bool(stored_password == val_password)



from typing import Any, Dict, Optional, Sequence, Union

from fastel.utils import requests


def format_phone_number(raw: str) -> str:
    pn = ""
    if raw:
        if raw[0] == "0":
            pn = "+886 " + raw[1:]
        elif raw[0] == "+":
            pn = raw
        else:
            pn = "+" + raw
    return pn


class Notifier:
    _stage: str
    _service_url: str
    _service_credentials: str

    def __init__(
        self,
        stage: str,
        client_id: str,
        client_secret: str,
    ):
        self._stage = stage
        self._service_url = (
            "https://notification.revtel-api.com/v4"
            if stage == "prod"
            else "https://notification-stg.revtel-api.com/v4"
        )
        self._service_credentials = (
            f"client_id={client_id}&client_secret={client_secret}"
        )

    def send_sms(self, target: str, title: str, message: str) -> Any:
        """
        Send a sms to target with message
        """

        result = requests.post(
            f"{self._service_url}/sms/send?{self._service_credentials}",
            json={
                "subject": title,  # TODO: check AWS api if subject is used in sms
                "phone": format_phone_number(target),
                "message": message,
            },
        )
        return result.json()

    def send_email(
        self, sender: str, target: Union[str, Sequence[str]], subject: str, html: str
    ) -> Any:
        """
        Send email(s) to target(s) with the subject and html content
        """

        result = requests.post(
            f"{self._service_url}/email/extra/send/{sender}?{self._service_credentials}",
            json={"to": target, "subject": subject, "html": html},
        )
        return result.json()

    def send_template_email(
        self,
        sender: str,
        target: Union[str, Sequence[str]],
        data: Dict[str, Any],
    ) -> Any:
        """
        Send email(s) to target(s) with content built from template using data

        sender: the sender name (`template` is a field in a sender document)
        target: the email(s) you want to send to
        template_data: the dict contains key-value pairs you'd like to pass to the sender's template
        """

        result = requests.post(
            f"{self._service_url}/email/send/{sender}?{self._service_credentials}",
            json={"to": target, "data": data},
        )
        return result.json()

    def push_to_user(
        self,
        user: str,
        subject: str,
        title: str,
        body: str,
        subtitle: Optional[str],
        data: Optional[Dict[str, Any]],
    ) -> Any:
        """
        Send push notifications to a specific user
        """

        result = requests.post(
            f"{self._service_url}/notification/publish/user/{user}?{self._service_credentials}",
            json={
                "subject": subject,
                "title": title,
                "body": body,
                "subtitle": subtitle if subtitle else "",
                "data": data if data else {},
            },
        )
        return result.json()

    def push_to_topic(
        self,
        topic: str,
        subject: str,
        title: str,
        body: str,
        subtitle: Optional[str],
        data: Optional[Dict[str, Any]],
    ) -> Any:
        """
        Send push notifications to all users subscribing a specific topic
        """

        result = requests.post(
            f"{self._service_url}/notification/publish/topic/{topic}?{self._service_credentials}",
            json={
                "subject": subject,
                "title": title,
                "body": body,
                "subtitle": subtitle if subtitle else "",
                "data": data if data else {},
            },
        )
        return result.json()

    def push_to_all(
        self,
        subject: str,
        title: str,
        body: str,
        subtitle: Optional[str],
        data: Optional[Dict[str, Any]],
    ) -> Any:
        """
        Send push notifications to all users
        """

        result = requests.post(
            f"{self._service_url}/notification/publish/topic/public-topic?{self._service_credentials}",
            json={
                "subject": subject,
                "title": title,
                "body": body,
                "subtitle": subtitle if subtitle else "",
                "data": data if data else {},
            },
        )
        return result.json()



from typing import Any, Dict

from jwt.algorithms import get_default_algorithms
from jwt.api_jwt import decode, decode_complete

from fastel.config import SdkConfig

from .pem_coder import pem_decode


def decode_static(token: str, pem: bool = False) -> Dict[str, Any]:
    if pem:
        return pem_decode(token)
    unverified = decode_complete(token, options={"verify_signature": False})
    signing_key = None
    signing_keys = SdkConfig.public_key
    for key in signing_keys["keys"]:
        if key["kid"] == unverified["header"]["kid"]:
            signing_key = key
            break

    algo = get_default_algorithms()["RS256"]  # type: ignore
    signing_key = algo.from_jwk(signing_key)
    verified = decode(
        token, key=signing_key, algorithms=["RS256"], options={"verify_aud": False}
    )

    return verified



from typing import List, Literal, Optional

from pydantic import BaseModel


class EZItem(BaseModel):
    name: str
    quantity: float
    price: float
    amount: float


class IssueModel(BaseModel):
    TransNum: Optional[str]
    MerchantOrderNo: str
    Status: Literal["0", "1", "3"] = "1"
    CreateStatusTime: Optional[str]
    Category: Literal["B2B", "B2C"] = "B2C"
    BuyerName: str
    BuyerUBN: Optional[str]
    BuyerAddress: Optional[str]
    BuyerEmail: Optional[str]
    CarrierType: Optional[Literal["0", "1", "2"]]
    CarrierNum: Optional[str]
    LoveCode: Optional[int]
    PrintFlag: Literal["Y", "N"] = "Y"
    TaxType: Literal["1", "2", "3", "9"] = "1"
    TaxRate: int = 5  # 5%
    CustomsClearance: Optional[Literal["1", "2"]]
    Amt: float
    AmtSales: Optional[int]
    AmtZero: Optional[int]
    AmtFree: Optional[int]
    TaxAmt: float
    TotalAmt: float
    items: List[EZItem]


class QueryModel(BaseModel):
    SearchType: Optional[Literal["0", "1"]]
    MerchantOrderNo: str
    TotalAmt: str
    InvoiceNumber: str
    RandomNum: str
    DisplayFlag: Optional[Literal["1"]]


class VoidModel(BaseModel):
    InvoiceNumber: str
    InvalidReason: str



# jwt

from .decode_static import decode_static
from .jwt_encoder import JWTEncoder
from .jwt_payload import (
    ClientAttrDesc,
    DataValueDesc,
    ExpDesc,
    IatDesc,
    JtiDesc,
    JWTPayloadBuilder,
    PayloadDesc,
)
from .kms_encode import kms_encode as encode
from .pem_coder import pem_decode, pem_encode, pem_random_key

decode = decode_static

__all__ = [
    "encode",
    "decode",
    "decode_static",
    "PayloadDesc",
    "ClientAttrDesc",
    "DataValueDesc",
    "IatDesc",
    "JtiDesc",
    "ExpDesc",
    "JWTPayloadBuilder",
    "JWTEncoder",
    "pem_random_key",
    "pem_encode",
    "pem_decode",
]



from typing import Literal, Optional

from pydantic import BaseModel


class RefundModel(BaseModel):
    Amt: int
    MerchantOrderNo: str
    IndexType: Literal[1, 2] = 1
    TradeNo: str
    CloseType: Literal[1, 2] = 2
    Cancel: Optional[Literal[1]]



import functools
from typing import Any, Dict


def skip_warmup_call(func: Any) -> Any:
    @functools.wraps(func)
    def warmup_wrapper(event: Dict[str, Any], context: Any) -> Any:
        if event.get("source") == "serverless-plugin-warmup":
            return "WarmUp - Lambda is warm!"
        return func(event, context)

    return warmup_wrapper



"""revtel fastapi package"""

from fastel.authorizers import (
    ClientIdAuth,
    ClientSecretAuth,
    Credential,
    JWBaseAuth,
    StaticClientAuth,
    StaticJWTAuth,
    StaticSecretAuth,
)
from fastel.cart.exceptions import CartException, cart_exception_handler
from fastel.collections import set_collections
from fastel.exceptions import APIException, api_exception_handler

__version__ = "1.13.45"
__all__ = [
    "APIException",
    "api_exception_handler",
    "ClientIdAuth",
    "ClientSecretAuth",
    "Credential",
    "JWBaseAuth",
    "StaticClientAuth",
    "StaticSecretAuth",
    "StaticJWTAuth",
    "cart_exception_handler",
    "CartException",
    "set_collections",
]
